﻿using System;
using System.Collections.Generic;
using System.IO;

namespace SynologyCertificateExporter.Services
{
    internal class CertificateExporter
    {
        private readonly string _synoDirectory;
        private readonly string _outputDirectory;

        private List<string> Filenames = new List<string>()
        {
            "cert.pem",
            "chain.pem",
            "privkey.pem"
        };

        internal CertificateExporter(string synoDirectory, string outputDirectory)
        {
            _synoDirectory = synoDirectory;
            _outputDirectory = outputDirectory;
        }

        internal void Export()
        {
            var infoJsonPath = _synoDirectory + "/INFO";
            var fileContent = File.ReadAllText(infoJsonPath);

            var certificates = SynologyInfoJsonParser.Parse(fileContent);

            CreateDirectoryIfNotExisting(_outputDirectory);

            foreach (var certificate in certificates)
            {
                var exportDirectoryName = !string.IsNullOrEmpty(certificate.Description) ? certificate.Description : certificate.Name;
                var exportDirectory = _outputDirectory + "/" + exportDirectoryName;

                CreateDirectoryIfNotExisting(exportDirectory);

                foreach (var fileName in Filenames)
                {
                    var cpFrom = _synoDirectory + "/" + certificate.Name + "/" + fileName;
                    var cpTo = exportDirectory + "/" + fileName;

                    if (File.Exists(cpTo))
                    {
                        var cpFromFileInfo = new FileInfo(cpFrom);
                        var cpToFileInfo = new FileInfo(cpTo);

                        if (cpFromFileInfo.CreationTime != cpToFileInfo.CreationTime)
                        {
                            Console.WriteLine($"Copy file from {cpFrom} to {cpTo} with overwrite because creationtime changed");

                            File.Copy(cpFrom, cpTo, true);
                        }
                        else
                        {
                            Console.WriteLine($"Skipping copy from {cpFrom} to {cpTo}");
                        }
                    }
                    else
                    {
                        Console.WriteLine($"Copy file from {cpFrom} to {cpTo}");
                        File.Copy(cpFrom, cpTo);
                    }
                }
            }
        }

        private void CreateDirectoryIfNotExisting(string path)
        {
            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);
        }
    }
}
